import time

from PyQt6.QtCore import QObject, pyqtSignal, QRunnable
from fontTools.misc.cliTools import makeOutputFileName
from fontTools.varLib.instancer import instantiateVariableFont, OverlapMode
from pathvalidate import sanitize_filename

from DRConverter.Lib.converters.options import Var2StaticOptions
from DRConverter.Lib.font.VFont import VariableFont


class WorkerSignals(QObject):
    progress = pyqtSignal(str)
    finished = pyqtSignal()


class JobRunner_vf2i(QRunnable):
    def __init__(self, files):
        super().__init__()
        self.files = files
        self.options = Var2StaticOptions()
        self.signals = WorkerSignals()
        self.is_killed = False

    def run(self) -> None:
        count = 0
        for file in self.files:
            count += 1
            self.signals.progress.emit(f"\nConverting file {count} of {len(self.files)}: {file}")

            try:
                variable_font = VariableFont(file, recalcTimestamp=self.options.recalc_timestamp)
                instances = variable_font.get_instances()
                if len(instances) == 0:
                    self.signals.progress.emit(f"ERROR: no static instances found.")
                    break

                if self.options.update_name_table:
                    if "STAT" not in variable_font:
                        self.options.update_name_table = False
                        self.signals.progress.emit("WARNING: Cannot update name table if there is no STAT table.")
                    if not hasattr(variable_font["STAT"], "AxisValueArray"):
                        self.options.update_name_table = False
                        self.signals.progress.emit(
                            "WARNING: Cannot update name table if there are no STAT Axis Values."
                        )

                instance_count = 0
                for instance in instances:
                    t = time.time()
                    instance_count += 1
                    self.signals.progress.emit(f"\nExporting instance {instance_count} of {len(instances)}")

                    try:
                        static_instance = instantiateVariableFont(
                            varfont=variable_font,
                            axisLimits=instance.coordinates,
                            inplace=False,
                            overlap=OverlapMode.REMOVE_AND_IGNORE_ERRORS,
                            optimize=True,
                            updateFontNames=self.options.update_name_table,
                        )

                        if "cvar" in static_instance:
                            del static_instance["cvar"]

                        if self.options.cleanup:
                            name_ids_to_delete = (
                                variable_font.get_var_name_ids_to_delete() if self.options.cleanup else []
                            )
                            static_instance.name_table.del_names(name_ids=name_ids_to_delete)

                            if "STAT" in static_instance:
                                del static_instance["STAT"]

                            static_instance.reorder_ui_name_ids()

                        static_instance_name = sanitize_filename(variable_font.get_instance_file_name(instance))
                        static_instance_extension = static_instance.get_real_extension()
                        output_file = makeOutputFileName(
                            static_instance_name,
                            extension=static_instance_extension,
                            outputDir=self.options.output_dir,
                            overWrite=self.options.overwrite,
                        )
                        static_instance.save(output_file)
                        message = f"{output_file} saved\nElapsed time: {round(time.time() - t, 3)} seconds"

                    except Exception as e:
                        message = f"FAIL: {e}"

                    self.signals.progress.emit(message)

                    if self.is_killed:
                        break

            except Exception as e:
                message = f"FAIL: {e}"
                self.signals.progress.emit(message)

        self.signals.finished.emit()



# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created by: PyQt6 UI code generator 6.5.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1000, 750)
        font = QtGui.QFont()
        font.setPointSize(9)
        MainWindow.setFont(font)
        self.centralwidget = QtWidgets.QWidget(parent=MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_Central = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_Central.setObjectName("gridLayout_Central")
        self.frameMiddle = QtWidgets.QFrame(parent=self.centralwidget)
        self.frameMiddle.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frameMiddle.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frameMiddle.setObjectName("frameMiddle")
        self.gridLayout_Middle = QtWidgets.QGridLayout(self.frameMiddle)
        self.gridLayout_Middle.setObjectName("gridLayout_Middle")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout_Middle.addLayout(self.gridLayout, 1, 0, 1, 1)
        self.labelFiles = QtWidgets.QLabel(parent=self.frameMiddle)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(True)
        font.setWeight(75)
        self.labelFiles.setFont(font)
        self.labelFiles.setObjectName("labelFiles")
        self.gridLayout_Middle.addWidget(self.labelFiles, 0, 0, 1, 1)
        self.gridLayout_Central.addWidget(self.frameMiddle, 1, 0, 1, 1)
        self.frameTop = QtWidgets.QFrame(parent=self.centralwidget)
        self.frameTop.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frameTop.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frameTop.setObjectName("frameTop")
        self.gridLayout_Top = QtWidgets.QGridLayout(self.frameTop)
        self.gridLayout_Top.setObjectName("gridLayout_Top")
        self.lineEdit_SourceFolder = QtWidgets.QLineEdit(parent=self.frameTop)
        self.lineEdit_SourceFolder.setObjectName("lineEdit_SourceFolder")
        self.gridLayout_Top.addWidget(self.lineEdit_SourceFolder, 0, 1, 1, 1)
        self.lineEdit_DestinationFolder = QtWidgets.QLineEdit(parent=self.frameTop)
        self.lineEdit_DestinationFolder.setObjectName("lineEdit_DestinationFolder")
        self.gridLayout_Top.addWidget(self.lineEdit_DestinationFolder, 1, 1, 1, 1)
        self.pushButton_BrowseSourceFolder = QtWidgets.QPushButton(parent=self.frameTop)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.pushButton_BrowseSourceFolder.setFont(font)
        self.pushButton_BrowseSourceFolder.setObjectName("pushButton_BrowseSourceFolder")
        self.gridLayout_Top.addWidget(self.pushButton_BrowseSourceFolder, 0, 2, 1, 1)
        self.label_DestinationFolder = QtWidgets.QLabel(parent=self.frameTop)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(True)
        font.setItalic(False)
        font.setWeight(75)
        self.label_DestinationFolder.setFont(font)
        self.label_DestinationFolder.setObjectName("label_DestinationFolder")
        self.gridLayout_Top.addWidget(self.label_DestinationFolder, 1, 0, 1, 1)
        self.label_SourceFolder = QtWidgets.QLabel(parent=self.frameTop)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(True)
        font.setWeight(75)
        self.label_SourceFolder.setFont(font)
        self.label_SourceFolder.setObjectName("label_SourceFolder")
        self.gridLayout_Top.addWidget(self.label_SourceFolder, 0, 0, 1, 1)
        self.pushButton_ConversionOptions = QtWidgets.QPushButton(parent=self.frameTop)
        self.pushButton_ConversionOptions.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.pushButton_ConversionOptions.setFont(font)
        self.pushButton_ConversionOptions.setObjectName("pushButton_ConversionOptions")
        self.gridLayout_Top.addWidget(self.pushButton_ConversionOptions, 2, 2, 1, 1)
        self.pushButton_BrowseDestinationFolder = QtWidgets.QPushButton(parent=self.frameTop)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.pushButton_BrowseDestinationFolder.setFont(font)
        self.pushButton_BrowseDestinationFolder.setObjectName("pushButton_BrowseDestinationFolder")
        self.gridLayout_Top.addWidget(self.pushButton_BrowseDestinationFolder, 1, 2, 1, 1)
        self.comboBox_ConversionProfile = QtWidgets.QComboBox(parent=self.frameTop)
        self.comboBox_ConversionProfile.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.comboBox_ConversionProfile.setFont(font)
        self.comboBox_ConversionProfile.setObjectName("comboBox_ConversionProfile")
        self.comboBox_ConversionProfile.addItem("")
        self.comboBox_ConversionProfile.addItem("")
        self.comboBox_ConversionProfile.addItem("")
        self.comboBox_ConversionProfile.addItem("")
        self.comboBox_ConversionProfile.addItem("")
        self.comboBox_ConversionProfile.addItem("")
        self.comboBox_ConversionProfile.addItem("")
        self.gridLayout_Top.addWidget(self.comboBox_ConversionProfile, 2, 1, 1, 1)
        self.label_ConversionProfile = QtWidgets.QLabel(parent=self.frameTop)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(True)
        font.setWeight(75)
        self.label_ConversionProfile.setFont(font)
        self.label_ConversionProfile.setObjectName("label_ConversionProfile")
        self.gridLayout_Top.addWidget(self.label_ConversionProfile, 2, 0, 1, 1)
        self.pushButton_Start = QtWidgets.QPushButton(parent=self.frameTop)
        self.pushButton_Start.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.pushButton_Start.setFont(font)
        self.pushButton_Start.setObjectName("pushButton_Start")
        self.gridLayout_Top.addWidget(self.pushButton_Start, 2, 3, 1, 1)
        self.gridLayout_Central.addWidget(self.frameTop, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(parent=MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionOpen_files = QtGui.QAction(parent=MainWindow)
        self.actionOpen_files.setObjectName("actionOpen_files")
        self.actionOpen_folder = QtGui.QAction(parent=MainWindow)
        self.actionOpen_folder.setObjectName("actionOpen_folder")
        self.actionExit = QtGui.QAction(parent=MainWindow)
        self.actionExit.setObjectName("actionExit")

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.lineEdit_SourceFolder, self.pushButton_BrowseSourceFolder)
        MainWindow.setTabOrder(self.pushButton_BrowseSourceFolder, self.lineEdit_DestinationFolder)
        MainWindow.setTabOrder(self.lineEdit_DestinationFolder, self.pushButton_BrowseDestinationFolder)
        MainWindow.setTabOrder(self.pushButton_BrowseDestinationFolder, self.comboBox_ConversionProfile)
        MainWindow.setTabOrder(self.comboBox_ConversionProfile, self.pushButton_ConversionOptions)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Font Converter"))
        self.labelFiles.setText(_translate("MainWindow", "Files"))
        self.pushButton_BrowseSourceFolder.setText(_translate("MainWindow", "Browse"))
        self.label_DestinationFolder.setText(_translate("MainWindow", "Destination folder"))
        self.label_SourceFolder.setText(_translate("MainWindow", "Source folder"))
        self.pushButton_ConversionOptions.setText(_translate("MainWindow", "Options"))
        self.pushButton_BrowseDestinationFolder.setText(_translate("MainWindow", "Browse"))
        self.comboBox_ConversionProfile.setCurrentText(_translate("MainWindow", "Select conversion profile"))
        self.comboBox_ConversionProfile.setItemText(0, _translate("MainWindow", "Select conversion profile"))
        self.comboBox_ConversionProfile.setItemText(1, _translate("MainWindow", "Web to SFNT"))
        self.comboBox_ConversionProfile.setItemText(2, _translate("MainWindow", "SFNT to Web"))
        self.comboBox_ConversionProfile.setItemText(3, _translate("MainWindow", "CFF to TrueType"))
        self.comboBox_ConversionProfile.setItemText(4, _translate("MainWindow", "TrueType to CFF"))
        self.comboBox_ConversionProfile.setItemText(5, _translate("MainWindow", "Variable to static"))
        self.comboBox_ConversionProfile.setItemText(6, _translate("MainWindow", "TTC to SFNT"))
        self.label_ConversionProfile.setText(_translate("MainWindow", "Conversion profile"))
        self.pushButton_Start.setText(_translate("MainWindow", "Convert"))
        self.actionOpen_files.setText(_translate("MainWindow", "Open files"))
        self.actionOpen_folder.setText(_translate("MainWindow", "Open folder"))
        self.actionExit.setText(_translate("MainWindow", "Exit"))



# Form implementation generated from reading ui file 'sfnt2web_options_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.5.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_sfnt2webOptions_Dialog(object):
    def setupUi(self, sfnt2webOptions_Dialog):
        sfnt2webOptions_Dialog.setObjectName("sfnt2webOptions_Dialog")
        sfnt2webOptions_Dialog.resize(400, 300)
        font = QtGui.QFont()
        font.setPointSize(9)
        sfnt2webOptions_Dialog.setFont(font)
        sfnt2webOptions_Dialog.setSizeGripEnabled(False)
        sfnt2webOptions_Dialog.setModal(False)
        self.gridLayout = QtWidgets.QGridLayout(sfnt2webOptions_Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=sfnt2webOptions_Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.RestoreDefaults|QtWidgets.QDialogButtonBox.StandardButton.Save)
        self.buttonBox.setCenterButtons(False)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 5, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 434, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout.addItem(spacerItem, 4, 0, 1, 1)
        self.overwrite_checkBox = QtWidgets.QCheckBox(parent=sfnt2webOptions_Dialog)
        self.overwrite_checkBox.setObjectName("overwrite_checkBox")
        self.gridLayout.addWidget(self.overwrite_checkBox, 2, 0, 1, 1)
        self.outputFormat_label = QtWidgets.QLabel(parent=sfnt2webOptions_Dialog)
        self.outputFormat_label.setObjectName("outputFormat_label")
        self.gridLayout.addWidget(self.outputFormat_label, 0, 0, 1, 1)
        self.outputFormat_comboBox = QtWidgets.QComboBox(parent=sfnt2webOptions_Dialog)
        self.outputFormat_comboBox.setObjectName("outputFormat_comboBox")
        self.outputFormat_comboBox.addItem("")
        self.outputFormat_comboBox.addItem("")
        self.outputFormat_comboBox.addItem("")
        self.gridLayout.addWidget(self.outputFormat_comboBox, 1, 0, 1, 1)
        self.recalcTimestamp_checkBox = QtWidgets.QCheckBox(parent=sfnt2webOptions_Dialog)
        self.recalcTimestamp_checkBox.setObjectName("recalcTimestamp_checkBox")
        self.gridLayout.addWidget(self.recalcTimestamp_checkBox, 3, 0, 1, 1)

        self.retranslateUi(sfnt2webOptions_Dialog)
        self.buttonBox.accepted.connect(sfnt2webOptions_Dialog.accept) # type: ignore
        self.buttonBox.rejected.connect(sfnt2webOptions_Dialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(sfnt2webOptions_Dialog)
        sfnt2webOptions_Dialog.setTabOrder(self.outputFormat_comboBox, self.overwrite_checkBox)
        sfnt2webOptions_Dialog.setTabOrder(self.overwrite_checkBox, self.recalcTimestamp_checkBox)

    def retranslateUi(self, sfnt2webOptions_Dialog):
        _translate = QtCore.QCoreApplication.translate
        sfnt2webOptions_Dialog.setWindowTitle(_translate("sfnt2webOptions_Dialog", "SFNT to Web options"))
        self.overwrite_checkBox.setText(_translate("sfnt2webOptions_Dialog", "Overwrite existing files"))
        self.outputFormat_label.setText(_translate("sfnt2webOptions_Dialog", "Output format"))
        self.outputFormat_comboBox.setItemText(0, _translate("sfnt2webOptions_Dialog", "WOFF + WOFF2"))
        self.outputFormat_comboBox.setItemText(1, _translate("sfnt2webOptions_Dialog", "WOFF"))
        self.outputFormat_comboBox.setItemText(2, _translate("sfnt2webOptions_Dialog", "WOFF2"))
        self.recalcTimestamp_checkBox.setText(_translate("sfnt2webOptions_Dialog", "Recalc timestamp"))



import time

from PyQt6.QtCore import QObject, pyqtSignal, QRunnable
from fontTools.misc.cliTools import makeOutputFileName
from fontTools.ttLib import TTCollection

from DRConverter.Lib.converters.options import TTCollectionToSFNTOptions


class WorkerSignals(QObject):
    progress = pyqtSignal(str)
    finished = pyqtSignal()


class JobRunner_ttc2sfnt(QRunnable):
    def __init__(self, files):
        super().__init__()
        self.files = files
        self.options = TTCollectionToSFNTOptions()
        self.signals = WorkerSignals()
        self.is_killed = False

    def run(self) -> None:
        count = 0
        for file in self.files:
            count += 1
            t = time.time()

            self.signals.progress.emit(f"\nConverting file {count} of {len(self.files)}: {file}")

            try:
                ttc = TTCollection(file)
                for font in ttc.fonts:
                    font.recalcTimestamp = self.options.recalc_timestamp
                    file_name = font["name"].getDebugName(6)
                    extension = ".otf" if font.sfntVersion == "OTTO" else ".ttf"
                    output_file = makeOutputFileName(
                        file_name,
                        outputDir=self.options.output_dir,
                        extension=extension,
                        overWrite=self.options.overwrite,
                    )
                    font.save(output_file)
                    self.signals.progress.emit(f"{output_file} saved")

                self.signals.progress.emit(f"{len(ttc.fonts)} fonts saved in {round(time.time() - t, 3)} seconds")

            except Exception as e:
                message = f"FAIL: {e}"
                self.signals.progress.emit(message)

            if self.is_killed:
                break

        self.signals.finished.emit()



import time

from PyQt6.QtCore import QObject, pyqtSignal, QRunnable
from fontTools.misc.cliTools import makeOutputFileName
from fontTools.pens.cu2quPen import Cu2QuPen
from fontTools.pens.ttGlyphPen import TTGlyphPen
from fontTools.ttLib import TTLibError, newTable

from DRConverter.Lib.font.Font import Font
from DRConverter.Lib.converters.options import CFFToTrueTypeOptions


class WorkerSignals(QObject):
    progress = pyqtSignal(str)
    finished = pyqtSignal()


class JobRunner_otf2ttf(QRunnable):
    def __init__(self, files):
        super().__init__()
        self.files = files
        self.options = CFFToTrueTypeOptions()
        self.signals = WorkerSignals()
        self.is_killed = False

    def run(self) -> None:
        count = 0
        for file in self.files:
            t = time.time()
            count += 1
            self.signals.progress.emit(f"\nConverting file {count} of {len(self.files)}: {file}")

            try:
                font = Font(file, recalcTimestamp=self.options.recalc_timestamp)

                converter = CFFToTrueType(font=font)
                converter.options.max_err = self.options.max_err
                ttf_font = converter.run()

                output_file = makeOutputFileName(
                    file, outputDir=self.options.output_dir, overWrite=self.options.overwrite, extension=".ttf"
                )
                ttf_font.save(output_file)

                message = f"{output_file} saved\nElapsed time: {round(time.time() - t, 3)} seconds"

            except Exception as e:
                message = f"FAIL: {e}"

            self.signals.progress.emit(message)

            if self.is_killed:
                break

        self.signals.finished.emit()


class CFFToTrueType(object):
    def __init__(self, font: Font):
        self.font = font
        self.options = CFFToTrueTypeOptions()

    def run(self):
        if self.font.sfntVersion != "OTTO":
            raise TTLibError("Not a OpenType font (bad sfntVersion)")
        assert "CFF " in self.font

        glyphOrder = self.font.getGlyphOrder()

        self.font["loca"] = newTable("loca")
        self.font["glyf"] = glyf = newTable("glyf")
        glyf.glyphOrder = glyphOrder
        glyf.glyphs = self.glyphs_to_quadratic(glyphs=self.font.getGlyphSet())
        del self.font["CFF "]
        if "VORG" in self.font:
            del self.font["VORG"]
        glyf.compile(self.font)
        self.update_hmtx(glyf)

        self.font["maxp"] = maxp = newTable("maxp")
        maxp.tableVersion = 0x00010000
        maxp.maxZones = 1
        maxp.maxTwilightPoints = 0
        maxp.maxStorage = 0
        maxp.maxFunctionDefs = 0
        maxp.maxInstructionDefs = 0
        maxp.maxStackElements = 0
        maxp.maxSizeOfInstructions = 0
        maxp.maxComponentElements = max(
            len(g.components if hasattr(g, "components") else []) for g in glyf.glyphs.values()
        )
        maxp.compile(self.font)

        post = self.font["post"]
        post.formatType = self.options.post_format
        post.extraNames = []
        post.mapping = {}
        post.glyphOrder = glyphOrder
        try:
            post.compile(self.font)
        except OverflowError:
            post.formatType = 3

        self.font.sfntVersion = "\000\001\000\000"
        return self.font

    def update_hmtx(self, glyf):
        hmtx = self.font["hmtx"]
        for glyphName, glyph in glyf.glyphs.items():
            if hasattr(glyph, "xMin"):
                hmtx[glyphName] = (hmtx[glyphName][0], glyph.xMin)

    def glyphs_to_quadratic(self, glyphs):
        quadGlyphs = {}
        for gname in glyphs.keys():
            glyph = glyphs[gname]
            ttPen = TTGlyphPen(glyphs)
            cu2quPen = Cu2QuPen(ttPen, max_err=self.options.max_err, reverse_direction=self.options.reverse_direction)
            glyph.draw(cu2quPen)
            quadGlyphs[gname] = ttPen.glyph()
        return quadGlyphs



import time

from afdko import checkoutlinesufo
from cffsubr import subroutinize
import pathops
from PyQt6.QtCore import QRunnable, QObject, pyqtSignal
from fontTools.fontBuilder import FontBuilder
from fontTools.misc.cliTools import makeOutputFileName
from fontTools.pens.qu2cuPen import Qu2CuPen
from fontTools.pens.t2CharStringPen import T2CharStringPen
from DRConverter.Lib.utils.subsetter import BaseSubsetter

from DRConverter.Lib.font.Font import Font
from DRConverter.Lib.converters.options import TrueTypeToCFFOptions


class WorkerSignals(QObject):
    progress = pyqtSignal(str)
    finished = pyqtSignal()


class JobRunner_ttf2otf(QRunnable):
    def __init__(self, files):
        super().__init__()
        self.files = files
        self.options = TrueTypeToCFFOptions()
        self.signals = WorkerSignals()
        self.is_killed = False

    def run(self) -> None:
        count = 0
        for file in self.files:
            t = time.time()
            count += 1

            try:
                self.signals.progress.emit(f"\nConverting file {count} of {len(self.files)}: {file}")

                source_font = Font(file, recalcTimestamp=self.options.recalc_timestamp)

                # Set tolerance as a ratio of unitsPerEm
                tolerance = self.options.tolerance / 1000 * source_font["head"].unitsPerEm

                ext = ".otf" if source_font.flavor is None else source_font.get_real_extension()
                suffix = "" if source_font.flavor is None else ".otf"
                output_file = makeOutputFileName(
                    file,
                    suffix=suffix,
                    extension=ext,
                    outputDir=self.options.output_dir,
                    overWrite=self.options.overwrite,
                )

                if self.options.safe_mode:
                    # Create a temporary OTF file with T2CharStringPen...
                    from DRConverter.Lib.converters.otf_to_ttf import CFFToTrueType

                    ttf2otf_converter_temp = TrueTypeToCFF(source_font)
                    ttf2otf_converter_temp.options.charstring_source = "t2"
                    ttf2otf_converter_temp.options.subroutinize = False
                    ttf2otf_converter_temp.options.purge_glyphs = self.options.remove_glyphs
                    temp_cff_font = ttf2otf_converter_temp.run()

                    # ... and convert it back to a temporary TTF file that will be used for conversion
                    otf_to_ttf_converter = CFFToTrueType(temp_cff_font)
                    # since the temp CFF font has many more points than needed, increase max_err from 1.0 to 2.0
                    otf_to_ttf_converter.options.max_err = 2.0
                    input_font = otf_to_ttf_converter.run()

                else:
                    input_font = source_font

                if self.options.scale_upm:
                    from fontTools.ttLib.scaleUpem import scale_upem

                    scale_upem(input_font, 1000)

                ttf2otf_converter = TrueTypeToCFF(font=input_font)
                ttf2otf_converter.options.charstring_source = "qu2cu"
                ttf2otf_converter.options.tolerance = tolerance
                ttf2otf_converter.options.subroutinize = self.options.subroutinize
                ttf2otf_converter.options.purge_glyphs = self.options.remove_glyphs
                ttf2otf_converter.options.check_outlines = self.options.check_outlines
                cff_font = ttf2otf_converter.run()

                cff_font.save(output_file)

                if self.options.check_outlines:
                    self.signals.progress.emit("Checking outlines with checkoutlinesufo")
                    checkoutlinesufo.run(args=[output_file, "--error-correction-mode", "--quiet-mode"])

                message = f"{output_file} saved\nElapsed time: {round(time.time() - t, 3)} seconds"

            except Exception as e:
                message = f"FAIL: {e}"

            self.signals.progress.emit(message)

            if self.is_killed:
                break

        self.signals.finished.emit()


class TrueTypeToCFF(object):
    def __init__(self, font: Font):
        self.font = font
        self.options = TrueTypeToCFFOptions()

    def run(self):
        if self.options.remove_glyphs:
            self.purge_glyphs()

        charstrings = {}

        if self.options.charstring_source == "qu2cu":
            self.font.decomponentize()
            try:
                charstrings = self.get_qu2cu_charstrings(tolerance=self.options.tolerance, all_cubic=True)
            except NotImplementedError:
                try:
                    charstrings = self.get_qu2cu_charstrings(tolerance=self.options.tolerance, all_cubic=False)
                except:
                    return

        if self.options.charstring_source == "t2":
            try:
                charstrings = self.get_t2_charstrings()
            except:
                return

        cff_font_info = self.get_cff_font_info()
        post_values = self.get_post_values()

        fb = FontBuilder(font=self.font)
        fb.isTTF = False
        for table in ["glyf", "cvt ", "loca", "fpgm", "prep", "gasp", "LTSH", "hdmx"]:
            if table in fb.font:
                del fb.font[table]

        fb.setupCFF(
            psName=self.font.name_table.getDebugName(6),
            charStringsDict=charstrings,
            fontInfo=cff_font_info,
            privateDict={},
        )
        fb.setupDummyDSIG()
        fb.setupMaxp()
        fb.setupPost(**post_values)

        if self.options.subroutinize:
            # cffsubr doesn't work with woff/woff2 fonts
            flavor = fb.font.flavor
            if flavor is not None:
                fb.font.flavor = None
            subroutinize(fb.font)
            fb.font.flavor = flavor

        return fb.font

    def get_cff_font_info(self) -> dict:
        """
        Setup CFF topDict

        :return: A dictionary of the font info.
        """

        font_revision = str(round(self.font["head"].fontRevision, 3)).split(".")
        major_version = str(font_revision[0])
        minor_version = str(font_revision[1]).ljust(3, "0")

        cff_font_info = dict(
            version=".".join([major_version, str(int(minor_version))]),
            FullName=self.font.name_table.getBestFullName(),
            FamilyName=self.font.name_table.getBestFamilyName(),
            ItalicAngle=self.font["post"].italicAngle,
            UnderlinePosition=self.font["post"].underlinePosition,
            UnderlineThickness=self.font["post"].underlineThickness,
            isFixedPitch=False if self.font["post"].isFixedPitch == 0 else True,
        )

        return cff_font_info

    def get_post_values(self) -> dict:
        post_info = dict(
            italicAngle=round(self.font["post"].italicAngle),
            underlinePosition=self.font["post"].underlinePosition,
            underlineThickness=self.font["post"].underlineThickness,
            isFixedPitch=self.font["post"].isFixedPitch,
            minMemType42=self.font["post"].minMemType42,
            maxMemType42=self.font["post"].maxMemType42,
            minMemType1=self.font["post"].minMemType1,
            maxMemType1=self.font["post"].maxMemType1,
        )
        return post_info

    def purge_glyphs(self):
        glyph_ids_to_remove = []
        for g in [".null", "NUL", "NULL", "uni0000", "CR", "nonmarkingreturn", "uni000D"]:
            try:
                glyph_ids_to_remove.append(self.font.getGlyphID(g))
            except KeyError:
                pass

        glyph_ids = [i for i in self.font.getReverseGlyphMap().values() if i not in glyph_ids_to_remove]
        if len(glyph_ids_to_remove) > 0:
            subsetter = BaseSubsetter(glyph_ids=glyph_ids)
            subsetter.subset(self.font)

    def get_qu2cu_charstrings(self, tolerance: float = 1, all_cubic: bool = True):
        charstrings = {}
        glyph_set = self.font.getGlyphSet()

        for k, v in glyph_set.items():
            # Correct contours direction and remove overlaps with pathops
            pathops_path = pathops.Path()
            pathops_pen = pathops_path.getPen(glyphSet=glyph_set)
            try:
                glyph_set[k].draw(pathops_pen)
                pathops_path.simplify()
            except TypeError:
                pass

            t2_pen = T2CharStringPen(v.width, glyphSet=glyph_set)
            qu2cu_pen = Qu2CuPen(t2_pen, max_err=tolerance, all_cubic=all_cubic, reverse_direction=False)
            pathops_path.draw(qu2cu_pen)

            charstring = t2_pen.getCharString()
            charstrings[k] = charstring

        return charstrings

    def get_t2_charstrings(self) -> dict:
        """
        Get CFF charstrings using T2CharStringPen

        :return: CFF charstrings.
        """
        charstrings = {}
        glyph_set = self.font.getGlyphSet()

        for k, v in glyph_set.items():
            # Draw the glyph with T2CharStringPen and get the charstring
            t2_pen = T2CharStringPen(v.width, glyphSet=glyph_set)
            glyph_set[k].draw(t2_pen)
            charstring = t2_pen.getCharString()
            charstrings[k] = charstring

        return charstrings



class Options(object):
    def __init__(self):
        self.recalc_timestamp = False
        self.output_dir = None
        self.overwrite = True


class WebToSFNTOptions(Options):
    def __init__(self):
        super().__init__()
        self.woff = True
        self.woff2 = True


class SFNTToWebOptions(Options):
    def __init__(self):
        super().__init__()
        self.woff = True
        self.woff2 = True


class CFFToTrueTypeOptions(Options):
    def __init__(self):
        super().__init__()
        self.max_err = 1.0
        self.post_format = 2.0
        self.reverse_direction = True


class Var2StaticOptions(Options):
    def __init__(self):
        super().__init__()
        self.cleanup = True
        self.update_name_table = True


class TrueTypeToCFFOptions(Options):
    def __init__(self):
        super().__init__()
        self.tolerance: float = 1.0
        self.charstring_source = "qu2cu"
        self.subroutinize = True
        self.check_outlines = False
        self.safe_mode = False
        self.remove_glyphs = True
        self.scale_upm = False


class TTCollectionToSFNTOptions(Options):
    def __init__(self):
        super().__init__()



import os

from fontTools.ttLib.tables._f_v_a_r import NamedInstance

from DRConverter.Lib.font.Font import Font


class VariableFont(Font):
    def __init__(self, file, recalcTimestamp=False):
        super().__init__(file=file, recalcTimestamp=recalcTimestamp)
        self.fvar_table = self["fvar"]

    def get_axes(self) -> list:
        return [axis for axis in self["fvar"].axes if axis.flags == 0]

    def get_instances(self) -> list:
        return [instance for instance in self["fvar"].instances]

    def get_var_name_ids_to_delete(self) -> list:
        name_ids_to_delete = [25]

        if "fvar" in self.keys():
            for axis in self.get_axes():
                name_ids_to_delete.append(axis.axisNameID)
            for instance in self.get_instances():
                if hasattr(instance, "subfamilyNameID"):
                    name_ids_to_delete.append(instance.subfamilyNameID)
                if hasattr(instance, "postscriptNameID"):
                    name_ids_to_delete.append(instance.postscriptNameID)

        if "STAT" in self.keys():
            if hasattr(self["STAT"].table, "DesignAxisRecord"):
                for axis in self["STAT"].table.DesignAxisRecord.Axis:
                    name_ids_to_delete.append(axis.AxisNameID)
            if hasattr(self["STAT"].table, "AxisValueArray") and self["STAT"].table.AxisValueArray is not None:
                for axis in self["STAT"].table.AxisValueArray.AxisValue:
                    name_ids_to_delete.append(axis.ValueNameID)

        name_ids_to_delete = [n for n in name_ids_to_delete if n > 24]
        name_ids_to_delete = sorted(list(set(name_ids_to_delete)))

        return name_ids_to_delete

    def get_instance_file_name(self, instance: NamedInstance) -> str:
        if hasattr(instance, "postscriptNameID") and instance.postscriptNameID < 65535:
            instance_file_name = self.name_table.getDebugName(instance.postscriptNameID)

        else:
            if hasattr(instance, "subfamilyNameID") and instance.subfamilyNameID > 0:
                subfamily_name = self.name_table.getDebugName(instance.subfamilyNameID)
            else:
                subfamily_name = "_".join([f"{k}_{v}" for k, v in instance.coordinates.items()])

            if self.name_table.getBestFamilyName() is not None:
                family_name = self.name_table.getBestFamilyName()
            else:
                family_name = os.path.splitext(os.path.basename(self.file))[0]

            instance_file_name = f"{family_name}-{subfamily_name}".replace(" ", "")

        return instance_file_name



# Form implementation generated from reading ui file 'output_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.5.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Output(object):
    def setupUi(self, Output):
        Output.setObjectName("Output")
        Output.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
        Output.resize(800, 600)
        font = QtGui.QFont()
        font.setPointSize(9)
        Output.setFont(font)
        Output.setAutoFillBackground(False)
        Output.setModal(False)
        self.verticalLayout = QtWidgets.QVBoxLayout(Output)
        self.verticalLayout.setObjectName("verticalLayout")
        self.plainTextEdit = QtWidgets.QPlainTextEdit(parent=Output)
        self.plainTextEdit.setEnabled(True)
        self.plainTextEdit.setAutoFillBackground(False)
        self.plainTextEdit.setReadOnly(True)
        self.plainTextEdit.setCenterOnScroll(False)
        self.plainTextEdit.setObjectName("plainTextEdit")
        self.verticalLayout.addWidget(self.plainTextEdit)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=Output)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Close|QtWidgets.QDialogButtonBox.StandardButton.Discard|QtWidgets.QDialogButtonBox.StandardButton.Reset)
        self.buttonBox.setCenterButtons(False)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Output)
        self.buttonBox.accepted.connect(Output.accept) # type: ignore
        self.buttonBox.rejected.connect(Output.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Output)

    def retranslateUi(self, Output):
        _translate = QtCore.QCoreApplication.translate
        Output.setWindowTitle(_translate("Output", "Output"))



# Form implementation generated from reading ui file 'otf2ttf_options_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.5.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_otf2ttfOptions_Dialog(object):
    def setupUi(self, otf2ttfOptions_Dialog):
        otf2ttfOptions_Dialog.setObjectName("otf2ttfOptions_Dialog")
        otf2ttfOptions_Dialog.resize(400, 300)
        font = QtGui.QFont()
        font.setPointSize(9)
        otf2ttfOptions_Dialog.setFont(font)
        otf2ttfOptions_Dialog.setSizeGripEnabled(False)
        otf2ttfOptions_Dialog.setModal(False)
        self.gridLayout = QtWidgets.QGridLayout(otf2ttfOptions_Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(parent=otf2ttfOptions_Dialog)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.tolerance_doubleSpinBox = QtWidgets.QDoubleSpinBox(parent=otf2ttfOptions_Dialog)
        self.tolerance_doubleSpinBox.setDecimals(1)
        self.tolerance_doubleSpinBox.setMaximum(2.5)
        self.tolerance_doubleSpinBox.setSingleStep(0.1)
        self.tolerance_doubleSpinBox.setObjectName("tolerance_doubleSpinBox")
        self.horizontalLayout.addWidget(self.tolerance_doubleSpinBox)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 434, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout.addItem(spacerItem1, 3, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=otf2ttfOptions_Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.RestoreDefaults|QtWidgets.QDialogButtonBox.StandardButton.Save)
        self.buttonBox.setCenterButtons(False)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 4, 0, 1, 1)
        self.recalcTimestamp_checkBox = QtWidgets.QCheckBox(parent=otf2ttfOptions_Dialog)
        self.recalcTimestamp_checkBox.setObjectName("recalcTimestamp_checkBox")
        self.gridLayout.addWidget(self.recalcTimestamp_checkBox, 2, 0, 1, 1)
        self.overwrite_checkBox = QtWidgets.QCheckBox(parent=otf2ttfOptions_Dialog)
        self.overwrite_checkBox.setObjectName("overwrite_checkBox")
        self.gridLayout.addWidget(self.overwrite_checkBox, 1, 0, 1, 1)

        self.retranslateUi(otf2ttfOptions_Dialog)
        self.buttonBox.accepted.connect(otf2ttfOptions_Dialog.accept) # type: ignore
        self.buttonBox.rejected.connect(otf2ttfOptions_Dialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(otf2ttfOptions_Dialog)
        otf2ttfOptions_Dialog.setTabOrder(self.tolerance_doubleSpinBox, self.overwrite_checkBox)
        otf2ttfOptions_Dialog.setTabOrder(self.overwrite_checkBox, self.recalcTimestamp_checkBox)

    def retranslateUi(self, otf2ttfOptions_Dialog):
        _translate = QtCore.QCoreApplication.translate
        otf2ttfOptions_Dialog.setWindowTitle(_translate("otf2ttfOptions_Dialog", "OTF to TTF options"))
        self.label.setText(_translate("otf2ttfOptions_Dialog", "Conversion tolerance"))
        self.recalcTimestamp_checkBox.setText(_translate("otf2ttfOptions_Dialog", "Recalc timestamp"))
        self.overwrite_checkBox.setText(_translate("otf2ttfOptions_Dialog", "Overwrite existing files"))



import time

from PyQt6.QtCore import QRunnable, pyqtSlot, QObject, pyqtSignal
from fontTools.misc.cliTools import makeOutputFileName

from DRConverter.Lib.converters.options import WebToSFNTOptions
from DRConverter.Lib.font.Font import Font


class WorkerSignals(QObject):
    progress = pyqtSignal(str)
    finished = pyqtSignal()


class JobRunner_wf2ft(QRunnable):
    def __init__(self, files):
        super().__init__()
        self.files = files
        self.options = WebToSFNTOptions()
        self.signals = WorkerSignals()
        self.is_killed = False

    @pyqtSlot()
    def run(self) -> None:
        count = 0
        for file in self.files:
            t = time.time()
            count += 1
            self.signals.progress.emit(f"\nConverting file {count} of {len(self.files)}: {file}")
            try:
                font = Font(file, recalcTimestamp=self.options.recalc_timestamp)

                if not font.flavor:
                    continue
                if not self.options.woff:
                    if font.flavor == "woff":
                        continue
                if not self.options.woff2:
                    if font.flavor == "woff2":
                        continue

                old_extension = font.get_real_extension()
                if self.options.woff and self.options.woff2:
                    suffix = old_extension
                else:
                    suffix = ""
                font.flavor = None
                new_extension = font.get_real_extension()
                output_file = makeOutputFileName(
                    file,
                    extension=new_extension,
                    suffix=suffix,
                    outputDir=self.options.output_dir,
                    overWrite=self.options.overwrite,
                )

                font.save(output_file)
                message = f"{output_file} saved\nElapsed time: {round(time.time() - t, 3)} seconds"

            except Exception as e:
                message = f"FAIL: {e}"

            if self.is_killed:
                break

            self.signals.progress.emit(message)

        self.signals.finished.emit()



import os
import sys
from pathlib import Path

from PyQt6 import QtCore, QtGui
from PyQt6.QtCore import QThreadPool
from PyQt6.QtWidgets import (
    QApplication,
    QMainWindow,
    QFileDialog,
    QDialog,
    QStyledItemDelegate,
    QListView,
    QDialogButtonBox, QFrame,
)
from fontTools.ttLib import TTCollection

from DRConverter.Lib.converters.options import (
    WebToSFNTOptions,
    TrueTypeToCFFOptions,
    SFNTToWebOptions,
    CFFToTrueTypeOptions,
    Var2StaticOptions,
    TTCollectionToSFNTOptions,
)
from DRConverter.Lib.converters.otf_to_ttf import JobRunner_otf2ttf
from DRConverter.Lib.converters.sfnt_to_web import JobRunner_ft2wf
from DRConverter.Lib.converters.ttc_to_sfnt import JobRunner_ttc2sfnt
from DRConverter.Lib.converters.ttf_to_otf import JobRunner_ttf2otf
from DRConverter.Lib.converters.variable_to_static import JobRunner_vf2i
from DRConverter.Lib.converters.web_to_sfnt import JobRunner_wf2ft
from DRConverter.Lib.font.Font import Font
from DRConverter.Lib.gui.MainWindow import Ui_MainWindow
from DRConverter.Lib.gui.dialogs.otf2ttf_options_dialog import Ui_otf2ttfOptions_Dialog
from DRConverter.Lib.gui.dialogs.output_dialog import Ui_Output
from DRConverter.Lib.gui.dialogs.sfnt2web_options_dialog import Ui_sfnt2webOptions_Dialog
from DRConverter.Lib.gui.dialogs.ttc2sfnt_options_dialog import Ui_ttc2sfntOptions_Dialog
from DRConverter.Lib.gui.dialogs.ttf2otf_options_dialog import Ui_ttf2otfOptions_Dialog
from DRConverter.Lib.gui.dialogs.vf2i_options_dialog import Ui_vf2iOptions_Dialog
from DRConverter.Lib.gui.dialogs.web2sfnt_options_dialog import Ui_web2sfntOptions_Dialog


class FontItem(object):
    file = None
    is_checked = None
    is_enabled = None
    extension = None
    is_sfnt = None
    is_webfont = None
    is_static = None
    is_variable = None
    is_cff = None
    is_ttf = None
    profiles = None


class FilesList(QListView):
    checked = QtCore.pyqtSignal(QtCore.QModelIndex)

    def __init__(self, *args, **kwargs):
        super(FilesList, self).__init__(*args, **kwargs)
        self.setItemDelegate(Delegate(self))


class Delegate(QStyledItemDelegate):
    def editorEvent(self, event, model, option, index):
        checked = index.data(QtCore.Qt.ItemDataRole.CheckStateRole)
        ret = QStyledItemDelegate.editorEvent(self, event, model, option, index)
        if checked != index.data(QtCore.Qt.ItemDataRole.CheckStateRole):
            self.parent().checked.emit(index)
        return ret


class TTF2OTFOptionsDialog(QDialog, Ui_ttf2otfOptions_Dialog):
    def __init__(self):
        super().__init__()
        self.setupUi(self)


class OTF2TTFOptionsDialog(QDialog, Ui_otf2ttfOptions_Dialog):
    def __init__(self):
        super().__init__()
        self.setupUi(self)


class SFNT2WEBOptionsDialog(QDialog, Ui_sfnt2webOptions_Dialog):
    def __init__(self):
        super().__init__()
        self.setupUi(self)


class WEB2SFNTOptionsDialog(QDialog, Ui_web2sfntOptions_Dialog):
    def __init__(self):
        super().__init__()
        self.setupUi(self)


class VF2IOptionsDialog(QDialog, Ui_vf2iOptions_Dialog):
    def __init__(self):
        super().__init__()
        self.setupUi(self)


class TTC2SFNTOptionsDialog(QDialog, Ui_ttc2sfntOptions_Dialog):
    def __init__(self):
        super().__init__()
        self.setupUi(self)


class OutputDialog(QDialog, Ui_Output):
    def __init__(self):
        super().__init__()
        self.setupUi(self)

        # Change the text os Reset and Discard buttons
        self.buttonBox.button(QDialogButtonBox.StandardButton.Reset).setText("Clear log")
        self.buttonBox.button(QDialogButtonBox.StandardButton.Discard).setText("Stop")

        # Disable the close button and remove the what's this button
        self.setWindowFlags(
            self.windowFlags()
            ^ QtCore.Qt.WindowType.WindowCloseButtonHint
            ^ QtCore.Qt.WindowType.WindowContextHelpButtonHint
        )


basedir = os.path.dirname(__file__)

try:
    from ctypes import windll  # Only exists on Windows.

    app_id = "ftcli.ftcli.fontconverter.0.1.0"
    windll.shell32.SetCurrentProcessExplicitAppUserModelID(app_id)
except ImportError:
    pass


class MainWindow(QMainWindow, Ui_MainWindow):
    def __init__(self):
        super().__init__()
        self.setupUi(self)
        self.fonts = [FontItem]
        self.input_folder = None
        self.output_folder = None
        self.last_folder = os.path.expanduser("~")
        self.center()

        # Set up the list view
        self.listView_Files = FilesList(self.centralwidget)
        self.listView_Files.setObjectName("listView_Files")
        self.gridLayout_Middle.addWidget(self.listView_Files)
        self.listView_Files.setFrameShape(QFrame.Shape.Box)
        self.files_model = QtGui.QStandardItemModel(self)
        self.item = QtGui.QStandardItem()
        self.listView_Files.setModel(self.files_model)
        self.listView_Files.checked.connect(self.item_checked)
        self.listView_Files.checked.connect(self.set_push_button_options_state)
        self.listView_Files.checked.connect(self.set_push_button_start_state)

        # Connect the Browse Source folder button
        self.pushButton_BrowseSourceFolder.clicked.connect(self.browse_source_folder)
        self.pushButton_BrowseSourceFolder.clicked.connect(self.get_font_items)
        self.pushButton_BrowseSourceFolder.clicked.connect(self.filter_font_items)
        self.pushButton_BrowseSourceFolder.clicked.connect(self.update_list_view)
        self.pushButton_BrowseSourceFolder.clicked.connect(self.set_push_button_start_state)
        self.pushButton_BrowseSourceFolder.clicked.connect(self.set_push_button_options_state)
        self.pushButton_BrowseSourceFolder.clicked.connect(self.set_combobox_conversion_profiles_state)

        # Connect the Browse Destination folder button
        self.pushButton_BrowseDestinationFolder.clicked.connect(self.browse_destination_folder)

        # Connect the Conversion Profile combo box
        self.comboBox_ConversionProfile.currentIndexChanged.connect(self.filter_font_items)
        self.comboBox_ConversionProfile.currentIndexChanged.connect(self.update_list_view)
        self.comboBox_ConversionProfile.currentIndexChanged.connect(self.set_push_button_options_state)
        self.comboBox_ConversionProfile.currentIndexChanged.connect(self.set_push_button_start_state)

        # Connect the Conversion Options tool button
        self.pushButton_ConversionOptions.clicked.connect(self.show_options_dialog)

        # Connect the Start button
        self.pushButton_Start.clicked.connect(self.start_process)
        self.pushButton_Start.clicked.connect(self.show_output_dialog)
        self.pushButton_Start.clicked.connect(self.get_font_items)
        self.pushButton_Start.clicked.connect(self.filter_font_items)
        self.pushButton_Start.clicked.connect(self.update_list_view)

        # Initialize the runner
        self.runner = None
        self.threadpool = QThreadPool()

        # Set up output dialog
        self.output_dialog = OutputDialog()
        self.output_dialog.buttonBox.button(QDialogButtonBox.StandardButton.Reset).clicked.connect(self.clear_log)
        self.output_dialog.buttonBox.button(QDialogButtonBox.StandardButton.Discard).clicked.connect(self.stop_runner)

        # Setup converter options
        self.ft2wf_options = SFNTToWebOptions()
        self.wf2ft_options = WebToSFNTOptions()
        self.ttf2otf_options = TrueTypeToCFFOptions()
        self.otf2ttf_options = CFFToTrueTypeOptions()
        self.vf2i_options = Var2StaticOptions()
        self.ttc2sfnt_options = TTCollectionToSFNTOptions()

        # Init wf2ft_dlg
        self.wf2ft_dlg = WEB2SFNTOptionsDialog()
        self.wf2ft_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(self.save_wf2ft_options)
        self.wf2ft_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(self.init_wf2ft_dialog)
        self.wf2ft_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(self.filter_font_items)
        self.wf2ft_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(self.update_list_view)
        self.wf2ft_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.reset_wf2ft_options)
        self.wf2ft_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.init_wf2ft_dialog)

        # Init ft2wf_dlg
        self.ft2wf_dlg = SFNT2WEBOptionsDialog()
        self.ft2wf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(self.save_ft2wf_options)
        self.ft2wf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(self.init_ft2wf_dialog)
        self.ft2wf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(self.filter_font_items)
        self.ft2wf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(self.update_list_view)
        self.ft2wf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.reset_ft2wf_options)
        self.ft2wf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.init_ft2wf_dialog)

        # Init tt2f2otf_dlg
        self.ttf2otf_dlg = TTF2OTFOptionsDialog()
        self.ttf2otf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(
            self.save_ttf2otf_options)
        self.ttf2otf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(
            self.init_ttf2otf_dialog)
        self.ttf2otf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.reset_ttf2otf_options)
        self.ttf2otf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.init_ttf2otf_dialog)

        # Init otf2ttf_dlg
        self.otf2ttf_dlg = OTF2TTFOptionsDialog()
        self.otf2ttf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(
            self.save_otf2ttf_options)
        self.otf2ttf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(
            self.init_otf2ttf_dialog)
        self.otf2ttf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.reset_otf2ttf_options)
        self.otf2ttf_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.init_otf2ttf_dialog)

        # Init vf2i_dlg
        self.vf2i_dlg = VF2IOptionsDialog()
        self.vf2i_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(self.save_vf2i_options)
        self.vf2i_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.reset_vf2i_options)
        self.vf2i_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.init_vf2i_dialog)

        # Init ttc2sfnt_dlg
        self.ttc2sfnt_dlg = TTC2SFNTOptionsDialog()
        self.ttc2sfnt_dlg.buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(
            self.save_ttc2sfnt_options)
        self.ttc2sfnt_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.reset_ttc2sfnt_options
        )
        self.ttc2sfnt_dlg.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(
            self.init_ttc2sfnt_dialog)

        # Show the app
        self.show()

    @QtCore.pyqtSlot(QtCore.QModelIndex)
    def item_checked(self, index):
        item = self.files_model.itemFromIndex(index)
        if item.checkState() == QtCore.Qt.CheckState.Checked:
            self.fonts[item.index().row()].is_checked = True
        else:
            self.fonts[item.index().row()].is_checked = False

    def browse_source_folder(self):
        dlg = QFileDialog(self)
        dlg.setWindowTitle("Browse source folder")
        dlg.setFileMode(QFileDialog.FileMode.Directory)
        dlg.setViewMode(QFileDialog.ViewMode.List)
        dlg.setDirectory(self.last_folder)

        if dlg.exec():
            self.input_folder = str(Path(dlg.selectedFiles()[0]))
            if self.input_folder:
                self.lineEdit_SourceFolder.setText(self.input_folder)
                self.last_folder = self.input_folder
                self.output_folder = self.input_folder
                self.lineEdit_DestinationFolder.setText(self.input_folder)

    def browse_destination_folder(self):
        dlg = QFileDialog(self)
        dlg.setWindowTitle("Browse destination folder")
        dlg.setFileMode(QFileDialog.FileMode.Directory)
        dlg.setViewMode(QFileDialog.ViewMode.List)
        dlg.setDirectory(self.last_folder)

        if dlg.exec():
            self.output_folder = str(Path(dlg.selectedFiles()[0]))
            if self.output_folder:
                self.lineEdit_DestinationFolder.setText(self.output_folder)

    def get_font_items(self):
        self.fonts = []
        if self.input_folder:
            files = [str(Path(os.path.join(self.input_folder, file))) for file in os.listdir(self.input_folder)]
            for file in files:
                try:
                    font = Font(file)
                    data = FontItem()
                    data.file = file
                    data.extension = font.get_real_extension()
                    data.is_variable = True if font.is_variable else False
                    data.is_checked = True
                    data.is_enabled = True
                    data.profiles = [1, 2, 3, 4, 5]
                    if font.flavor is None:  # SFNT font
                        data.profiles.remove(1)
                    else:  # Web font
                        data.profiles.remove(2)
                    if not self.wf2ft_options.woff2 and font.flavor == "woff2":
                        data.profiles.remove(1)
                    if not self.wf2ft_options.woff and font.flavor == "woff":
                        data.profiles.remove(1)
                    if font.is_true_type or font.is_variable:  # TTF or variable font
                        data.profiles.remove(3)
                    if font.is_cff or font.is_variable:  # CFF or variable font
                        data.profiles.remove(4)
                    if not font.is_variable:  # Static font
                        data.profiles.remove(5)

                    data.is_webfont = True if font.get_real_extension() in (".woff", ".woff2") else False
                    data.is_sfnt = False if font.get_real_extension() in (".woff", ".woff2") else True
                    data.is_cff = True if font.is_cff else False
                    data.is_ttf = True if font.is_true_type else False

                    self.fonts.append(data)

                except:
                    try:
                        TTCollection(file)
                        data = FontItem()
                        data.file = file
                        data.profiles = [6]
                        data.is_checked = True
                        data.is_enabled = True

                        self.fonts.append(data)

                    except:
                        pass

    def filter_font_items(self):
        for font in self.fonts:
            if self.comboBox_ConversionProfile.currentIndex() == 0:  # No profile selected
                font.is_enabled = False
                font.is_checked = False
            elif self.comboBox_ConversionProfile.currentIndex() in font.profiles:
                font.is_enabled = True
                font.is_checked = True
            else:
                font.is_enabled = False
                font.is_checked = False

    def update_list_view(self):
        self.files_model.clear()
        for font in self.fonts:
            item = QtGui.QStandardItem(str(os.path.basename(font.file)))
            item.setEditable(False)
            item.setCheckable(True)
            if self.comboBox_ConversionProfile.currentIndex() == 0:
                item.setCheckable(False)
                item.setEnabled(False)
            elif self.comboBox_ConversionProfile.currentIndex() in font.profiles:
                item.setCheckable(True)
                item.setEnabled(True)
                item.setCheckState(QtCore.Qt.CheckState.Checked)
            else:
                item.setCheckable(False)
                item.setEnabled(False)

            self.files_model.appendRow(item)

    def set_combobox_conversion_profiles_state(self):
        if len(self.fonts) > 0:
            self.comboBox_ConversionProfile.setEnabled(True)
        else:
            self.comboBox_ConversionProfile.setDisabled(True)
            self.comboBox_ConversionProfile.setCurrentIndex(0)

    def set_push_button_start_state(self):
        selected_fonts = [ft for ft in self.fonts if ft.is_checked]
        if len(selected_fonts) > 0 and self.comboBox_ConversionProfile.currentIndex() != 0:
            self.pushButton_Start.setEnabled(True)
        else:
            self.pushButton_Start.setDisabled(True)

    def set_push_button_options_state(self):
        if self.comboBox_ConversionProfile.currentIndex() == 0:
            self.pushButton_ConversionOptions.setDisabled(True)
        else:
            self.pushButton_ConversionOptions.setEnabled(True)

    def show_options_dialog(self):
        if self.comboBox_ConversionProfile.currentIndex() == 1:
            self.init_wf2ft_dialog()
            self.wf2ft_dlg.exec()

        elif self.comboBox_ConversionProfile.currentIndex() == 2:
            self.init_ft2wf_dialog()
            self.ft2wf_dlg.exec()

        elif self.comboBox_ConversionProfile.currentIndex() == 3:
            self.init_otf2ttf_dialog()
            self.otf2ttf_dlg.exec()

        elif self.comboBox_ConversionProfile.currentIndex() == 4:
            self.init_ttf2otf_dialog()
            self.ttf2otf_dlg.exec()

        elif self.comboBox_ConversionProfile.currentIndex() == 5:
            self.init_vf2i_dialog()
            self.vf2i_dlg.exec()

        elif self.comboBox_ConversionProfile.currentIndex() == 6:
            self.init_ttc2sfnt_dialog()
            self.ttc2sfnt_dlg.exec()

    def init_wf2ft_dialog(self):
        if self.wf2ft_options.woff and self.wf2ft_options.woff2:
            self.wf2ft_dlg.inputFormat_comboBox.setCurrentIndex(0)

        if self.wf2ft_options.woff and not self.wf2ft_options.woff2:
            self.wf2ft_dlg.inputFormat_comboBox.setCurrentIndex(1)

        if self.wf2ft_options.woff2 and not self.wf2ft_options.woff:
            self.wf2ft_dlg.inputFormat_comboBox.setCurrentIndex(2)

        if self.wf2ft_options.overwrite:
            self.wf2ft_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.wf2ft_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.wf2ft_options.recalc_timestamp:
            self.wf2ft_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.wf2ft_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

    def save_wf2ft_options(self):
        if self.wf2ft_dlg.inputFormat_comboBox.currentIndex() == 0:
            self.wf2ft_options.woff = True
            self.wf2ft_options.woff2 = True

        if self.wf2ft_dlg.inputFormat_comboBox.currentIndex() == 1:
            self.wf2ft_options.woff = True
            self.wf2ft_options.woff2 = False

        if self.wf2ft_dlg.inputFormat_comboBox.currentIndex() == 2:
            self.wf2ft_options.woff = False
            self.wf2ft_options.woff2 = True

        for font in self.fonts:
            if font.extension == ".woff":
                if not self.wf2ft_options.woff:
                    if 1 in font.profiles:
                        font.profiles.remove(1)
                else:
                    font.profiles.append(1)

            if font.extension == ".woff2":
                if not self.wf2ft_options.woff2:
                    if 1 in font.profiles:
                        font.profiles.remove(1)
                else:
                    font.profiles.append(1)

            font.profiles = list(set(font.profiles))

        self.wf2ft_options.overwrite = True if self.wf2ft_dlg.overwrite_checkBox.isChecked() else False

        self.wf2ft_options.recalc_timestamp = True if self.wf2ft_dlg.recalcTimestamp_checkBox.isChecked() else False

    def reset_wf2ft_options(self):
        self.wf2ft_options = WebToSFNTOptions()

    def init_ft2wf_dialog(self):
        if self.ft2wf_options.woff and not self.ft2wf_options.woff2:
            self.ft2wf_dlg.outputFormat_comboBox.setCurrentIndex(1)

        elif self.ft2wf_options.woff2 and not self.ft2wf_options.woff:
            self.ft2wf_dlg.outputFormat_comboBox.setCurrentIndex(2)

        else:
            self.ft2wf_dlg.outputFormat_comboBox.setCurrentIndex(0)

        if self.ft2wf_options.overwrite:
            self.ft2wf_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.ft2wf_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.ft2wf_options.recalc_timestamp:
            self.ft2wf_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.ft2wf_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

    def save_ft2wf_options(self):
        if self.ft2wf_dlg.outputFormat_comboBox.currentIndex() == 0:
            self.ft2wf_options.woff = True
            self.ft2wf_options.woff2 = True

        if self.ft2wf_dlg.outputFormat_comboBox.currentIndex() == 1:
            self.ft2wf_options.woff = True
            self.ft2wf_options.woff2 = False

        if self.ft2wf_dlg.outputFormat_comboBox.currentIndex() == 2:
            self.ft2wf_options.woff = False
            self.ft2wf_options.woff2 = True

        self.ft2wf_options.overwrite = True if self.ft2wf_dlg.overwrite_checkBox.isChecked() else False

        self.ft2wf_options.recalc_timestamp = True if self.ft2wf_dlg.recalcTimestamp_checkBox.isChecked() else False

    def reset_ft2wf_options(self):
        self.ft2wf_options = SFNTToWebOptions()

    def init_otf2ttf_dialog(self):
        self.otf2ttf_dlg.tolerance_doubleSpinBox.setValue(self.otf2ttf_options.max_err)

        if self.otf2ttf_options.recalc_timestamp:
            self.otf2ttf_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.otf2ttf_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.otf2ttf_options.overwrite:
            self.otf2ttf_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.otf2ttf_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

    def save_otf2ttf_options(self):
        self.otf2ttf_options.max_err = self.otf2ttf_dlg.tolerance_doubleSpinBox.value()

        if self.otf2ttf_dlg.recalcTimestamp_checkBox.isChecked():
            self.otf2ttf_options.recalc_timestamp = True
        else:
            self.otf2ttf_options.recalc_timestamp = False

        if self.otf2ttf_dlg.overwrite_checkBox.isChecked():
            self.otf2ttf_options.overwrite = True
        else:
            self.otf2ttf_options.overwrite = False

    def reset_otf2ttf_options(self):
        self.otf2ttf_options = CFFToTrueTypeOptions()

    def init_ttf2otf_dialog(self):
        self.ttf2otf_dlg.tolerance_doubleSpinBox.setValue(self.ttf2otf_options.tolerance)

        if self.ttf2otf_options.remove_glyphs:
            self.ttf2otf_dlg.removeGlyphs_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.ttf2otf_dlg.removeGlyphs_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.ttf2otf_options.subroutinize:
            self.ttf2otf_dlg.subroutinize_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.ttf2otf_dlg.subroutinize_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.ttf2otf_options.check_outlines:
            self.ttf2otf_dlg.checkOutlines_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.ttf2otf_dlg.checkOutlines_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.ttf2otf_options.safe_mode:
            self.ttf2otf_dlg.safeMode_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.ttf2otf_dlg.safeMode_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.ttf2otf_options.scale_upm:
            self.ttf2otf_dlg.scaleUPM_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.ttf2otf_dlg.scaleUPM_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.ttf2otf_options.recalc_timestamp:
            self.ttf2otf_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.ttf2otf_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.ttf2otf_options.overwrite:
            self.ttf2otf_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.ttf2otf_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

    def save_ttf2otf_options(self):
        self.ttf2otf_options.tolerance = self.ttf2otf_dlg.tolerance_doubleSpinBox.value()

        if self.ttf2otf_dlg.removeGlyphs_checkBox.isChecked():
            self.ttf2otf_options.remove_glyphs = True
        else:
            self.ttf2otf_options.remove_glyphs = False

        if self.ttf2otf_dlg.subroutinize_checkBox.isChecked():
            self.ttf2otf_options.subroutinize = True
        else:
            self.ttf2otf_options.subroutinize = False

        if self.ttf2otf_dlg.checkOutlines_checkBox.isChecked():
            self.ttf2otf_options.check_outlines = True
        else:
            self.ttf2otf_options.check_outlines = False

        if self.ttf2otf_dlg.safeMode_checkBox.isChecked():
            self.ttf2otf_options.safe_mode = True
        else:
            self.ttf2otf_options.safe_mode = False

        if self.ttf2otf_dlg.scaleUPM_checkBox.isChecked():
            self.ttf2otf_options.scale_upm = True
        else:
            self.ttf2otf_options.scale_upm = False

        if self.ttf2otf_dlg.recalcTimestamp_checkBox.isChecked():
            self.ttf2otf_options.recalc_timestamp = True
        else:
            self.ttf2otf_options.recalc_timestamp = False

        if self.ttf2otf_dlg.overwrite_checkBox.isChecked():
            self.ttf2otf_options.overwrite = True
        else:
            self.ttf2otf_options.overwrite = False

    def reset_ttf2otf_options(self):
        self.ttf2otf_options = TrueTypeToCFFOptions()

    def init_vf2i_dialog(self):
        if self.vf2i_options.update_name_table:
            self.vf2i_dlg.updateNameTable_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.vf2i_dlg.updateNameTable_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.vf2i_options.cleanup:
            self.vf2i_dlg.cleanupInstances_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.vf2i_dlg.cleanupInstances_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.vf2i_options.recalc_timestamp:
            self.vf2i_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.vf2i_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.vf2i_options.overwrite:
            self.vf2i_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.vf2i_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

    def save_vf2i_options(self):
        self.vf2i_options.cleanup = True if self.vf2i_dlg.cleanupInstances_checkBox.isChecked() else False

        self.vf2i_options.update_name_table = True if self.vf2i_dlg.updateNameTable_checkBox.isChecked() else False

        self.vf2i_options.recalc_timestamp = True if self.vf2i_dlg.recalcTimestamp_checkBox.isChecked() else False

        self.vf2i_options.overwrite = True if self.vf2i_dlg.overwrite_checkBox.isChecked() else False

    def reset_vf2i_options(self):
        self.vf2i_options = Var2StaticOptions()

    def init_ttc2sfnt_dialog(self):
        if self.ttc2sfnt_options.overwrite:
            self.ttc2sfnt_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.ttc2sfnt_dlg.overwrite_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

        if self.ttc2sfnt_options.recalc_timestamp:
            self.ttc2sfnt_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.ttc2sfnt_dlg.recalcTimestamp_checkBox.setCheckState(QtCore.Qt.CheckState.Unchecked)

    def save_ttc2sfnt_options(self):
        self.ttc2sfnt_options.overwrite = True if self.ttc2sfnt_dlg.overwrite_checkBox.isChecked() else False
        self.ttc2sfnt_options.recalc_timestamp = (
            True if self.ttc2sfnt_dlg.recalcTimestamp_checkBox.isChecked() else False
        )

    def reset_ttc2sfnt_options(self):
        self.ttc2sfnt_options = TTCollectionToSFNTOptions()

    def start_process(self):
        self.output_dialog.buttonBox.button(QDialogButtonBox.StandardButton.Reset).setDisabled(True)
        self.output_dialog.buttonBox.button(QDialogButtonBox.StandardButton.Discard).setEnabled(True)
        self.output_dialog.buttonBox.button(QDialogButtonBox.StandardButton.Close).setDisabled(True)

        files = [font.file for font in self.fonts if font.is_checked]

        # Web to SFNT
        if self.comboBox_ConversionProfile.currentIndex() == 1:
            self.runner = JobRunner_wf2ft(files)
            self.runner.options = self.wf2ft_options

        # SFNT to Web
        elif self.comboBox_ConversionProfile.currentIndex() == 2:
            self.runner = JobRunner_ft2wf(files)
            self.runner.options = self.ft2wf_options

        # CFF to TrueType
        elif self.comboBox_ConversionProfile.currentIndex() == 3:
            self.runner = JobRunner_otf2ttf(files)
            self.runner.options = self.otf2ttf_options

        # TrueType to CFF
        elif self.comboBox_ConversionProfile.currentIndex() == 4:
            self.runner = JobRunner_ttf2otf(files)
            self.runner.options = self.ttf2otf_options

        # Variable to static
        elif self.comboBox_ConversionProfile.currentIndex() == 5:
            self.runner = JobRunner_vf2i(files)
            self.runner.options = self.vf2i_options

        elif self.comboBox_ConversionProfile.currentIndex() == 6:
            self.runner = JobRunner_ttc2sfnt(files)
            self.runner.options = self.ttc2sfnt_options

        else:
            pass

        self.runner.options.output_dir = self.output_folder
        self.runner.signals.progress.connect(self.print_process_output)
        self.runner.signals.finished.connect(self.runner_finished)
        self.threadpool.thread().started.connect(self.runner_started)
        self.threadpool.start(self.runner)

    def runner_started(self):
        self.output_dialog.buttonBox.button().setDisabled(True)
        self.output_dialog.buttonBox.button(QDialogButtonBox.StandardButton.Discard).setEnabled(True)
        self.output_dialog.buttonBox.button(QDialogButtonBox.StandardButton.Close).setDisabled(True)

    def stop_runner(self):
        if self.runner:
            self.runner.is_killed = True

    def runner_finished(self):
        if self.runner.is_killed:
            self.output_dialog.plainTextEdit.appendPlainText("\nPROCESS ABORTED")
        else:
            self.output_dialog.plainTextEdit.appendPlainText("\nPROCESS COMPLETED")
        self.output_dialog.buttonBox.button(QDialogButtonBox.StandardButton.Reset).setEnabled(True)
        self.output_dialog.buttonBox.button(QDialogButtonBox.StandardButton.Discard).setDisabled(True)
        self.output_dialog.buttonBox.button(QDialogButtonBox.StandardButton.Close).setEnabled(True)
        self.runner = None

    def show_output_dialog(self):
        self.output_dialog.exec()

    def clear_log(self):
        self.output_dialog.plainTextEdit.clear()

    def print_process_output(self, s):
        self.output_dialog.plainTextEdit.appendPlainText(s)

    def center(self):
        qr = self.frameGeometry()
        cp = self.screen().availableGeometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())


def main():
    app = QApplication(sys.argv)
    app.setWindowIcon(QtGui.QIcon(os.path.join(basedir, "icons", "icon.ico")))
    w = MainWindow()
    w.setWindowTitle("DR Converter")
    w.show()
    app.exec()


if __name__ == "__main__":
    main()



from fontTools.ttLib.tables._n_a_m_e import (
    table__n_a_m_e,
    _MAC_LANGUAGE_CODES,
    _WINDOWS_LANGUAGE_CODES,
)


class TableName(table__n_a_m_e):
    def filter_namerecords(
        self,
        name_ids=None,
        platform_id=None,
        plat_enc_id=None,
        lang_id=None,
        lang_string=None,
    ):
        """
        It takes a list of name records, and returns a list of name records that match the given criteria

        :param name_ids: A list of name IDs to filter by
        :param platform_id: The platform ID of the name record
        :param plat_enc_id: The platform-specific encoding ID
        :param lang_id: The language ID of the name record
        :param lang_string: The string to search for in the name records
        :return: A list of name records.
        """
        filtered_names = self.names
        if name_ids is not None:
            filtered_names = [name for name in filtered_names if name.nameID in name_ids]
        if platform_id is not None:
            filtered_names = [name for name in filtered_names if name.platformID == platform_id]
        if plat_enc_id is not None:
            filtered_names = [name for name in filtered_names if name.platEncID == plat_enc_id]
        if lang_id is not None:
            filtered_names = [name for name in filtered_names if name.langID == lang_id]
        if lang_string is not None:
            mac_lang_id = _MAC_LANGUAGE_CODES.get(lang_string.lower())
            win_lang_id = _WINDOWS_LANGUAGE_CODES.get(lang_string.lower())
            filtered_names = [name for name in filtered_names if name.langID in (mac_lang_id, win_lang_id)]
        return filtered_names

    def del_names(self, name_ids, platform_id=None, language_string=None) -> None:
        """
        Deletes all name records that match the given name_ids, optionally filtering by platform_id and/or
        language_string.

        :param name_ids: A list of name IDs to delete
        :param platform_id: The platform ID of the name records to delete
        :param language_string: The language of the name records to delete
        """

        names = self.filter_namerecords(name_ids=name_ids, platform_id=platform_id, lang_string=language_string)

        for name in names:
            self.removeNames(name.nameID, name.platformID, name.platEncID, name.langID)



import os

from fontTools.pens.recordingPen import DecomposingRecordingPen
from fontTools.pens.ttGlyphPen import TTGlyphPen
from fontTools.ttLib.tables._f_v_a_r import NamedInstance
from fontTools.ttLib.ttFont import TTFont, registerCustomTableClass

from DRConverter.Lib.font.tables.name import TableName

registerCustomTableClass("name", "DRConverter.Lib.font.tables.name", "TableName")


class Font(TTFont):
    def __init__(self, file, recalcTimestamp=False):
        super().__init__(file=file, recalcTimestamp=recalcTimestamp)

        self.file = file
        self.name_table: TableName = self["name"]

    @property
    def is_cff(self) -> bool:
        return self.sfntVersion == "OTTO"

    @property
    def is_true_type(self) -> bool:
        return "glyf" in self

    @property
    def is_woff(self) -> bool:
        return self.flavor == "woff"

    @property
    def is_woff2(self) -> bool:
        return self.flavor == "woff2"

    @property
    def is_variable(self) -> bool:
        return "fvar" in self

    @property
    def is_static(self) -> bool:
        return "fvar" not in self

    def get_real_extension(self) -> str:
        if self.flavor is not None:
            return f".{self.flavor}"
        elif self.is_true_type:
            return ".ttf"
        elif self.is_cff:
            return ".otf"

    def decomponentize(self):
        if not self.is_true_type:
            return
        glyph_set = self.getGlyphSet()
        glyf_table = self["glyf"]
        dr_pen = DecomposingRecordingPen(glyph_set)
        tt_pen = TTGlyphPen(None)

        for glyph_name in self.glyphOrder:
            glyph = glyf_table[glyph_name]
            if not glyph.isComposite():
                continue

            dr_pen.value = []
            tt_pen.init()

            glyph.draw(dr_pen, glyf_table)
            dr_pen.replay(tt_pen)

            glyf_table[glyph_name] = tt_pen.glyph()

    # Variable fonts functions

    def get_ui_name_ids(self) -> list:
        """
        Returns a list of all the UI name IDs in the font's GSUB table

        :return: A list of integers.
        """
        ui_name_ids = []
        if "GSUB" not in self.keys():
            return []
        else:
            for record in self["GSUB"].table.FeatureList.FeatureRecord:
                if record.Feature.FeatureParams is not None:
                    ui_name_ids.append(record.Feature.FeatureParams.UINameID)
        return sorted(set(ui_name_ids))

    def reorder_ui_name_ids(self):
        """
        Takes the IDs of the UI names in the name table and reorders them to start at 256
        """

        if "GSUB" not in self:
            return
        ui_name_ids = self.get_ui_name_ids()
        for count, value in enumerate(ui_name_ids, start=256):
            for n in self["name"].names:
                if n.nameID == value:
                    n.nameID = count
            for record in self["GSUB"].table.FeatureList.FeatureRecord:
                if record.Feature.FeatureParams:
                    if record.Feature.FeatureParams.UINameID == value:
                        record.Feature.FeatureParams.UINameID = count

    def get_axes(self) -> list:
        return [axis for axis in self["fvar"].axes if axis.flags == 0]

    def get_instances(self) -> list:
        return [instance for instance in self["fvar"].instances]

    def get_var_name_ids_to_delete(self) -> list:
        name_ids_to_delete = [25]

        if "fvar" in self.keys():
            for axis in self.get_axes():
                name_ids_to_delete.append(axis.axisNameID)
            for instance in self.get_instances():
                if hasattr(instance, "subfamilyNameID"):
                    name_ids_to_delete.append(instance.subfamilyNameID)
                if hasattr(instance, "postscriptNameID"):
                    name_ids_to_delete.append(instance.postscriptNameID)

        if "STAT" in self.keys():
            if hasattr(self["STAT"].table, "DesignAxisRecord"):
                for axis in self["STAT"].table.DesignAxisRecord.Axis:
                    name_ids_to_delete.append(axis.AxisNameID)
            if hasattr(self["STAT"].table, "AxisValueArray") and self["STAT"].table.AxisValueArray is not None:
                for axis in self["STAT"].table.AxisValueArray.AxisValue:
                    name_ids_to_delete.append(axis.ValueNameID)

        name_ids_to_delete = [n for n in name_ids_to_delete if n > 24]
        name_ids_to_delete = sorted(list(set(name_ids_to_delete)))

        return name_ids_to_delete

    def get_static_instance_file_name(self, instance: NamedInstance) -> str:
        if hasattr(instance, "postscriptNameID") and instance.postscriptNameID < 65535:
            instance_file_name = self["name"].getDebugName(instance.postscriptNameID)

        else:
            if hasattr(instance, "subfamilyNameID") and instance.subfamilyNameID > 0:
                subfamily_name = self["name"].getDebugName(instance.subfamilyNameID)
            else:
                subfamily_name = "_".join([f"{k}_{v}" for k, v in instance.coordinates.items()])

            if self["name"].getBestFamilyName() is not None:
                family_name = self["name"].getBestFamilyName()
            else:
                family_name = os.path.splitext(os.path.basename(self.file))[0]

            instance_file_name = f"{family_name}-{subfamily_name}".replace(" ", "")

        return instance_file_name



import setuptools

setuptools.setup(
    name="dr-converter",
    version="1.0.0",
    description="DR Font Converter.",
    author="Tom Joad",
    packages=setuptools.find_packages(),
    include_package_data=True,
    entry_points={"console_scripts": ["drconv=DRConverter.dr_converter:main"]},
    install_requires=[
        "fonttools>=4.39.3",
        "skia-pathops>=0.7.4",
        "brotli>=1.0.9",
        "cffsubr>=0.2.9.post1",
        "afdko==3.9.5",
        "pathvalidate>=2.5.2",
        "PyQt6==6.5.0",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    license="MIT",
    python_requires=">=3.7",
    zip_safe=False,
)



from fontTools.subset import Subsetter


class BaseSubsetter(Subsetter):
    def __init__(self, glyph_ids: list):
        super().__init__()
        self.glyph_ids = glyph_ids
        self.options.drop_tables = []
        self.options.passthrough_tables = True
        self.options.name_IDs = "*"
        self.options.name_legacy = True
        self.options.name_languages = "*"
        self.options.layout_features = "*"
        self.options.hinting = False
        self.options.notdef_glyph = True
        self.options.notdef_outline = True
        self.glyph_ids_requested = self.glyph_ids



# Form implementation generated from reading ui file 'ttf2otf_options_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.5.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_ttf2otfOptions_Dialog(object):
    def setupUi(self, ttf2otfOptions_Dialog):
        ttf2otfOptions_Dialog.setObjectName("ttf2otfOptions_Dialog")
        ttf2otfOptions_Dialog.resize(400, 300)
        font = QtGui.QFont()
        font.setPointSize(9)
        ttf2otfOptions_Dialog.setFont(font)
        ttf2otfOptions_Dialog.setSizeGripEnabled(False)
        ttf2otfOptions_Dialog.setModal(False)
        self.gridLayout = QtWidgets.QGridLayout(ttf2otfOptions_Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.overwrite_checkBox = QtWidgets.QCheckBox(parent=ttf2otfOptions_Dialog)
        self.overwrite_checkBox.setObjectName("overwrite_checkBox")
        self.gridLayout.addWidget(self.overwrite_checkBox, 8, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(parent=ttf2otfOptions_Dialog)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.tolerance_doubleSpinBox = QtWidgets.QDoubleSpinBox(parent=ttf2otfOptions_Dialog)
        self.tolerance_doubleSpinBox.setDecimals(1)
        self.tolerance_doubleSpinBox.setMaximum(2.5)
        self.tolerance_doubleSpinBox.setSingleStep(0.1)
        self.tolerance_doubleSpinBox.setObjectName("tolerance_doubleSpinBox")
        self.horizontalLayout.addWidget(self.tolerance_doubleSpinBox)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 0, 1, 1)
        self.safeMode_checkBox = QtWidgets.QCheckBox(parent=ttf2otfOptions_Dialog)
        self.safeMode_checkBox.setObjectName("safeMode_checkBox")
        self.gridLayout.addWidget(self.safeMode_checkBox, 5, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 434, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout.addItem(spacerItem1, 10, 0, 1, 1)
        self.subroutinize_checkBox = QtWidgets.QCheckBox(parent=ttf2otfOptions_Dialog)
        self.subroutinize_checkBox.setObjectName("subroutinize_checkBox")
        self.gridLayout.addWidget(self.subroutinize_checkBox, 2, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=ttf2otfOptions_Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.RestoreDefaults|QtWidgets.QDialogButtonBox.StandardButton.Save)
        self.buttonBox.setCenterButtons(False)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 11, 0, 1, 1)
        self.removeGlyphs_checkBox = QtWidgets.QCheckBox(parent=ttf2otfOptions_Dialog)
        self.removeGlyphs_checkBox.setObjectName("removeGlyphs_checkBox")
        self.gridLayout.addWidget(self.removeGlyphs_checkBox, 1, 0, 1, 1)
        self.recalcTimestamp_checkBox = QtWidgets.QCheckBox(parent=ttf2otfOptions_Dialog)
        self.recalcTimestamp_checkBox.setObjectName("recalcTimestamp_checkBox")
        self.gridLayout.addWidget(self.recalcTimestamp_checkBox, 9, 0, 1, 1)
        self.scaleUPM_checkBox = QtWidgets.QCheckBox(parent=ttf2otfOptions_Dialog)
        self.scaleUPM_checkBox.setObjectName("scaleUPM_checkBox")
        self.gridLayout.addWidget(self.scaleUPM_checkBox, 7, 0, 1, 1)
        self.checkOutlines_checkBox = QtWidgets.QCheckBox(parent=ttf2otfOptions_Dialog)
        self.checkOutlines_checkBox.setObjectName("checkOutlines_checkBox")
        self.gridLayout.addWidget(self.checkOutlines_checkBox, 4, 0, 1, 1)

        self.retranslateUi(ttf2otfOptions_Dialog)
        self.buttonBox.accepted.connect(ttf2otfOptions_Dialog.accept) # type: ignore
        self.buttonBox.rejected.connect(ttf2otfOptions_Dialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(ttf2otfOptions_Dialog)
        ttf2otfOptions_Dialog.setTabOrder(self.tolerance_doubleSpinBox, self.removeGlyphs_checkBox)
        ttf2otfOptions_Dialog.setTabOrder(self.removeGlyphs_checkBox, self.subroutinize_checkBox)
        ttf2otfOptions_Dialog.setTabOrder(self.subroutinize_checkBox, self.safeMode_checkBox)
        ttf2otfOptions_Dialog.setTabOrder(self.safeMode_checkBox, self.overwrite_checkBox)
        ttf2otfOptions_Dialog.setTabOrder(self.overwrite_checkBox, self.recalcTimestamp_checkBox)

    def retranslateUi(self, ttf2otfOptions_Dialog):
        _translate = QtCore.QCoreApplication.translate
        ttf2otfOptions_Dialog.setWindowTitle(_translate("ttf2otfOptions_Dialog", "TTF to OTF options"))
        self.overwrite_checkBox.setText(_translate("ttf2otfOptions_Dialog", "Overwrite existing files"))
        self.label.setText(_translate("ttf2otfOptions_Dialog", "Conversion tolerance (0-2.5)"))
        self.safeMode_checkBox.setText(_translate("ttf2otfOptions_Dialog", "Safe mode"))
        self.subroutinize_checkBox.setText(_translate("ttf2otfOptions_Dialog", "Subroutinize converted files"))
        self.removeGlyphs_checkBox.setText(_translate("ttf2otfOptions_Dialog", "Remove .null and CR glyphs"))
        self.recalcTimestamp_checkBox.setText(_translate("ttf2otfOptions_Dialog", "Recalc timestamp"))
        self.scaleUPM_checkBox.setText(_translate("ttf2otfOptions_Dialog", "Scale UPM to 1000"))
        self.checkOutlines_checkBox.setToolTip(_translate("ttf2otfOptions_Dialog", "Performs optional outline quality checks and removes overlaps with afdko.checkoutlinesufo"))
        self.checkOutlines_checkBox.setText(_translate("ttf2otfOptions_Dialog", "Check outlines after conversion"))



import time

from PyQt6.QtCore import QObject, pyqtSignal, QRunnable
from fontTools.misc.cliTools import makeOutputFileName

from DRConverter.Lib.font.Font import Font
from DRConverter.Lib.converters.options import SFNTToWebOptions


class WorkerSignals(QObject):
    progress = pyqtSignal(str)
    finished = pyqtSignal()


class JobRunner_ft2wf(QRunnable):
    def __init__(self, files):
        super().__init__()
        self.files = files
        self.options = SFNTToWebOptions()
        self.signals = WorkerSignals()
        self.is_killed = False

    def run(self) -> None:
        output_flavors = ["woff", "woff2"]
        if not self.options.woff:
            output_flavors.remove("woff")
        if not self.options.woff2:
            output_flavors.remove("woff2")

        count = 0
        for file in self.files:
            t = time.time()
            count += 1
            self.signals.progress.emit(f"\nConverting file {count} of {len(self.files)}: {file}")
            try:
                font = Font(file, recalcTimestamp=self.options.recalc_timestamp)

                if font.flavor is not None:
                    continue

                if self.options.woff:
                    converter = SFNTToWeb(font=font, flavor="woff")
                    web_font = converter.run()
                    extension = web_font.get_real_extension()
                    output_file = makeOutputFileName(
                        file, extension=extension, outputDir=self.options.output_dir, overWrite=self.options.overwrite
                    )
                    web_font.save(output_file, reorderTables=False)
                    message = f"{output_file} saved\nElapsed time: {round(time.time() - t, 3)} seconds"
                    self.signals.progress.emit(message)

                if self.options.woff2:
                    converter = SFNTToWeb(font=font, flavor="woff2")
                    web_font = converter.run()
                    extension = web_font.get_real_extension()
                    output_file = makeOutputFileName(
                        file, extension=extension, outputDir=self.options.output_dir, overWrite=self.options.overwrite
                    )
                    web_font.save(output_file, reorderTables=False)
                    message = f"{output_file} saved\nElapsed time: {round(time.time() - t, 3)} seconds"
                    self.signals.progress.emit(message)

            except Exception as e:
                message = f"FAIL: {e}"
                self.signals.progress.emit(message)

            if self.is_killed:
                break

        self.signals.finished.emit()


class SFNTToWeb(object):
    def __init__(self, font: Font, flavor: str):
        self.font = font
        self.flavor = flavor

    def run(self) -> Font:
        self.font.flavor = self.flavor
        return self.font
